% Lippi - Schivardi   -  Created by Francesco, May 2010   -- revised  nov 2012
% This code solves for the values of (q,R,ls) that produce a given pair (PHI, X. var(eta) )
% 
% This code estimates 7 params:  q, R1, R2, R3, R4, lm  , ls
% -- using normalized data on 9 observables
%
disp('   '); disp(' ******  ');
disp('Running Code Estimate_theta7   by Lippi-Schivardi');
disp('Given the primitives (beta, Ls, rho) and data vectors (X, phi) '); 
disp('Find ( q, R, lm ) consistent with the data (NORMALIZED TO 1984 ,  SIZE AND TECH LEVEL)');
disp('Estimate 7 params:  q, R1, R2, R3, R4, lm , ls  ');

clear all; close all; 

% Calibrate value of beta  , rho, LS
nperiods = 5; % n. of periods to become senior
bbeta    = 0.98^nperiods   ; % over a 4 year period (our unit)
rho_y    = 0.11    ;    % senior manager probability of exiting per year (Fabiano estimate is 0.099333)
rho      = 1 - (1-rho_y)^nperiods ;    % probability of senior manager dying over a 4 year period (our unit)
% ls       = 0.36;   %  this will be estimated

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set max number of iteration for simplex
maxit = 2000;
usesimplex=1;
alreadynorm  = 1;  %  choose whether estimate use raw data or normalzied data
withfirmsize = 1;  %  choose whether estimate use firm-size in the ctrl set  
    
if alreadynorm==1
    data =load('datamatlab4points.txt');
    ctrl_S=data(:,1);
    nobs_S=data(:,6);
    if withfirmsize==0;
        disp('Estimating model w/o  firm-size ctrl ');
        lnX_normz_KT=data(:,12);
        phi_normz_KT=data(:,10);
    else
        disp('Estimating model with firm-size ctrl ');
        lnX_normz_KT=data(:,13);
         
        phi_normz_KT=data(:,11);
        
     %   adjusted phi measure for gvt firms   
     %   phi_normz_KT(4) = 0.590; 
        
    end
    Sigma2 = [.35^2 .29^2];
    Lm0    = lnX_normz_KT(1);
    
else
    
    data    = load('datamatlab.txt');
    year_vec     = data(:,1);
    sect_vec     = data(:,2);
    ctrl_vec     = data(:,3);
    phi_data     = data(:,4);
    X_data       = data(:,5);
    nobs_vec     = data(:,8); % Number of firms in each cell
    lnX_data     = data(:,9);
    lnEmpl_data  = data(:,10); %MMM
    
    %  NOW normalize all data to sector 1 year 1984
    
    % Control dummies for family(1),Group(2),foreign(3), government(4)
    indx_fam = find(data(:,3)==1 );
    indx_grp = find(data(:,3)==2 );
    indx_for = find(data(:,3)==3 );
    indx_gvt = find(data(:,3)==4 );
    
    % tech sector dummy
    DDsect = zeros(length(sect_vec),3);%
    DDsect( find(sect_vec==2),1)=1;% medium low
    DDsect( find(sect_vec==3),2)=1;% medium high
    DDsect( find(sect_vec==4),3)=1;% high tech
    
    % Control dummies for family(1),Group(2),foreign(3), government(4)
    DDctrl = zeros(length(sect_vec),3);
    DDctrl( find(ctrl_vec==1),1)=1;
    DDctrl( find(ctrl_vec==2),2)=1;
    DDctrl( find(ctrl_vec==3),3)=1;
  
    
    %  NOW normalize all data to sector 1 year 1984
    [B] = ols(lnX_data,[ones(length(lnX_data),1),year_vec,DDsect,lnEmpl_data]);
    [D] = ols(phi_data,[ones(length(lnX_data),1),year_vec,DDsect,lnEmpl_data]);
    
    LmsB    = [0 B.beta([3 4 5])'];
    LmsD    = [0 D.beta([3 4 5])'];
    
    for j = 1 : length(data)
        lnX_normz(j) = lnX_data(j) - B.beta(2) * (year_vec(j)-1984) - LmsB(sect_vec(j))...
            -  B.beta(6)*(lnEmpl_data(j) - lnEmpl_data(1)) ;
        phi_normz(j) =  phi_data(j) - D.beta(2) * (year_vec(j)-1984) - LmsD(sect_vec(j))...
            -  D.beta(6)*(lnEmpl_data(j) - lnEmpl_data(1));
        phi_mean(j) =  phi_data(j) ; % - D.beta(2) * (year_vec(j)-1984) - LmsD(sect_vec(j)) ; 
        
    end
    
    % aggregate cells at control level (4 obs)
    
    ii=0;
    for  kk=1:4
        % recall data(:,3)  is Kontrol type family(1),Group(2),foreign(3),government (4)
        indx_KT = find(data(:,3)==kk);
        
        ii=ii+1;
        
        ctrl_S(ii) = kk;
        nobs_S(ii) = sum(nobs_vec(indx_KT));
        % computed mean (weighted by nobs)
        lnX_normz_KT(ii) = lnX_normz(indx_KT)*nobs_vec(indx_KT)/sum(nobs_vec(indx_KT));
        phi_normz_KT(ii) = phi_normz(indx_KT)*nobs_vec(indx_KT)/sum(nobs_vec(indx_KT));
        
        phi_normz_mean_KT(ii)=phi_mean(indx_KT)*nobs_vec(indx_KT)/sum(nobs_vec(indx_KT));
               
    end
    
    % Estimated sigma2 - regression fabiano nov 09
    [STATS1] = ols(lnX_data,[ones(length(lnX_data),1),year_vec,DDsect,lnEmpl_data,DDctrl]);
    [STATS2] = ols(phi_data,[ones(length(lnX_data),1),year_vec,DDsect,lnEmpl_data,DDctrl]);
    Sigma2 = [STATS1.sige     STATS2.sige]; % Variance of innovation in above regress
    %[0.43117^2     0.29039^2];  % Fabiano's estimate on individual data
    
    % Guess initial values for q, R (in proportion to mu)
    Lm0  =  B.beta(1)+B.beta(2)*1984+ B.beta(6)*lnEmpl_data(1);
end% IF ALREADYNORM


%  Input data on estimated  std_eta; Kontrol type family(1),conglomerate(2),foreign(3),government(4)
%  data generated by fabiano ,  reported in Table in the paper
std_eta_KT=[ 0.30  0.37  0.43  0.39   ]; 

% Now starts structural estimation using the data constructed above

q0   =  [0.2] ;
R0   =   exp(0.2*Lm0 );
RR0  =   ones(1,4)*R0;



% Trasform initial vec of estimated params so that they are bounded
ls0=0.36;  
%theta0  = [q0./(1-q0) R0 R0 R0 Lm0 R0 ls0  ];

theta0 = [2.8753    6.0584    3.7379   51.6271    1.5834    4.1653    0.4132];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load prevoius estimates
% estimates = load('theta5_est_13nov09');
% theta0 = estimates.theta;
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sstar0_vec = ones(1,length(data(:,1)))*2*exp(Lm0+ls0^2/2);
save sstar0_vec
load sstar0_vec

% Min errors of fit
if usesimplex==1;
    disp('Using simplex');
    % Use simplex method
    % theta0 = theta_est; options:   display =  iter/final
    
    options = optimset('Display','iter','MaxIter',maxit,'LargeScale','off');
 [theta_est,FVAL,EXITFLAG, OUTPUT] =  fminsearch( @(theta) fun_estimate_theta7(theta,...
        bbeta,rho,Sigma2,lnX_normz_KT,phi_normz_KT,std_eta_KT, ctrl_S,nobs_S),[theta0],options );
end

theta=abs(theta_est);

% compute standard errors
[ InfoM, InfoM_BHHH, GG_hat , dphi_dt, dlnX_dt , err ]= ...
    fun_Infomatrix_theta7(theta_est,bbeta,rho,Sigma2,lnX_normz_KT,phi_normz_KT,std_eta_KT, ctrl_S,nobs_S) ;


% save output
e1=num2str(withfirmsize,1);
%e2=num2str(ls,3);
e3=num2str(date);
e4=num2str(OUTPUT.iterations);

filename=['theta7_norm_size',e1,'_date',e3];

% save output 
save([filename,'.mat']);

q_vec_est  = abs(theta(1))./(1 + abs(theta(1)));
R_vec_est  = [abs(theta(2)) abs(theta(3)) abs(theta(6))  abs(theta(4)) ] ;
Lm   = abs(theta(5)); % Lm in sector 1 in year 1984
ls   = abs(theta(7)) ; 


Vars_theta_BH = diag(inv(InfoM_BHHH));
tstats_BH     = theta./sqrt(Vars_theta_BH');
Vars_theta_M  = diag(inv(InfoM));
tstats_M      =  theta./sqrt(Vars_theta_M');

diary([filename,'.txt']);


disp('  ');
if withfirmsize==0;
    disp('R, q , ls Estimates using normalized data;  controls for G(): year , sector');
else
    disp('R, q , ls Estimates using normalized data;  controls for G(): year , sector , firm size');
end;
disp('  ');
disp(' PRIMITIVE PARAMS ');
disp(['BETA   ',num2str(bbeta,3),';  rho (5 years)', num2str(rho,3) ]);
disp('  ');
disp(['The estimated theta* is']);
disp(num2str(theta,4));
% disp('  ');
% disp(['  t-stats from BHHH']);
% disp(num2str(tstats_BH,4));
disp('  ');
disp([' t-stats using asympt. ']);
disp(num2str(tstats_M,4));


disp('  ');
disp(' ESTIMATES ');
disp(' MSE of fit for log X , phi  and std_eta');
disp(['        ',num2str(std(err))]);
disp('  ');
disp(' Minimized value of objective function (normalized by:  3 * nobs)');
disp(['      FVAL:        ',num2str(FVAL)]);


disp('  ');
disp(' IMPLIED ECONOMIC PARAMS');
disp(['est. q                  (theta1)   ',num2str(q_vec_est), '  ,  t-stat  ',num2str(tstats_M(1)) ]);
disp('  ');
disp(['est. R  for family      (theta2)   ',num2str(theta(2)), ' ,  t-stat  ', num2str(tstats_M(2))] );
disp(['est. R  for conglomerat (theta3)   ',num2str(theta(3)), ' ,  t-stat  ',num2str(tstats_M(3))] );
disp(['est. R  for government  (theta4)   ',num2str(theta(4)), ' ,  t-stat  ',num2str(tstats_M(4))] );
disp(['est. R  for foreign     (theta6)   ',num2str(theta(6)), ' ,  t-stat  ',num2str(tstats_M(6))] );
disp('  ');
disp(['est. Lm  (theta5)  ',num2str(Lm) , ' ,  t-stat  ', num2str(tstats_M(5))]);
disp(['est. ls  (theta7)  ',num2str(ls) , ' ,  t-stat  ', num2str(tstats_M(7))]);


diary off

% Draw a figure with data and fitted points

close;
figure(1)
ph=plot(lnX_normz_KT,phi_normz_KT,'ko'); hold on
set(ph,'LineWidth',4);

q    = q_vec_est;
lm = Lm;
mu = exp(lm+ls^2/2); %  using the theory: exp(lm+ls^2/2)
quadlim =  1.2* fzero(@(x) 1-logncdf(x,lm,ls),exp(lm+ls^2/2));

vline(log(mu),'r','log (mu)')

disp(['Solving the model loci for the figure ...wait.... ']);


for i=1:length(R_vec_est)
    R = R_vec_est(i);
    %        disp(['computing R = ',num2str(R)]);
    
    
   [X,phi, mu , Xo,  std_Xo, std_X_unc, std_eps, sstar] =...
                    fun_Solve_Xphi_new(R,q,rho,lm,ls,bbeta,quadlim);
    
    
    % St st fraction of old -- phi in (0,1)
    
    std_eps_fit(i,1) = std_eps/mu; 
    phi_fit(i,1) = phi  ;
    lnX_fit(i,1) = log(X) ;
    
end % for i

ph=plot(lnX_fit,phi_fit,'r+'); hold on;
set(ph,'LineWidth',4);


% now compute the whole locus
R_vec= [0:1.3*(max(R_vec_est))/50:1.3*(max(R_vec_est))];

for i=1:length(R_vec)
    
    R = R_vec(i);
    %disp(['computing R = ',num2str(R)]);
    
   [X,phi, mu , Xo,  std_Xo, std_X_unc, std_eps, sstar] =...
                    fun_Solve_Xphi_new(R,q,rho,lm,ls,bbeta,quadlim);
    
    % St st fraction of old -- phi in (0,1)
    std_eps_vec(i,1) = std_eps/mu; 
    phi_vec(i,1) = phi  ;
    lnX_vec(i,1) = log(X) ;
    
end % for i

ph=plot(lnX_vec,phi_vec,'b:');
set(ph,'LineWidth',3);
lh=legend('data o ,   fit + ,  theory locus : ');
% set(lh,'LineWidth',4);

xh = xlabel('Normalized  Log  TFP ($ \log \ X $ )');
set(xh,'FontSize',16,'Interpreter','Latex');
 
yh = ylabel('Share of senior executives ( $\phi$ )  '); 
set(yh,'FontSize',16,'Interpreter','Latex');


% orient landscape
%  print -depsc /Users/francescolippi/Documents/LIPPONE/RESEARCH/i_miei_paper/managers/figures/fig_est_theta6

lm=theta(5);
ls=theta(7);

q  = q_vec_est;
mu = exp(lm+ls^2/2);
bigR=[theta(2) theta(3) theta(4) theta(6)];
quadlim = fzero(@(x) 1-logncdf(x,lm,ls),mu);

for i=0:4
    if i==0;
        R=0;
    else
        R=bigR(i);
    end
    
    if R>quadlim;
       R=quadlim;
    end
    
   [X,phi, mu , Xo,  std_Xo, std_X_unc, std_eps, sstar] =...
                     fun_Solve_Xphi_new(R,q,rho,lm,ls,bbeta,quadlim);
    % toc
    % compute Xo|q=0,1 and probability of replacing the old manager
    Xoq1=quad(@(s) fun_INT_G(s,lm,ls),max(1e-8,sstar-R),quadlim)/(1-logncdf(max(1e-8,sstar-R),lm,ls));
    Xoq0=quad(@(s) fun_INT_G(s,lm,ls),min(sstar,quadlim),quadlim)/(1-logncdf(min(sstar,quadlim),lm,ls));
    fired_overall=F_cdf(sstar,q,R,lm,ls);
    fired_q1=logncdf(max(1e-8,sstar-R),lm,ls);
    fired_q0=logncdf(min(sstar,quadlim),lm,ls);
    
    results=[R sstar max(0,sstar-R) log(X) log(Xo) log(Xoq0) log(Xoq1) phi fired_overall fired_q0 fired_q1]';
    
    if i==0
        allres=results;
    else
        allres=[allres,results];
    end
end


% return

diary on;

disp('Results');
disp(['R == 0  R-family  R-conglom   R-govt R-foreign']);
disp(num2str(allres,3));
disp('Rows content: R  sstar  xstar log(X) log(Xo) log(Xoq0) log(Xoq1) phi fired_mean fired_q0 fired_q1')
diary off;
