function [ InfoM, InfoM_BHHH, GG_hat, dphi_dt, dlnX_dt , err ]=...
         fun_Infomatrix_theta7(theta,bbeta,rho,Sigma2,lnX_normz,phi_data,std_eta_KT,ctrl_vec,nobs_vec) 
% This subroutine computes the information matrix evaluated at theta_est;
% As the notes explain, for this model the n,m element of the information matrix InfoM is given by :
%
%  InfoM(n,m) = Sum j=1,2 (1/ sigma_j^2 ) (dlog f^j(theta)/d theta_n)*(dlog f^j(theta)/dtheta_m)
%
% where  j=1,2,  are the two observables log(X) and phi .
%        theta = (q, R_vec, Lm0 ) the vector of parameters
%        f^j(theta) is the model prediction of variable j given vector of parameters theta
%        sigma_j^2 the variance of the meas error for variable j
%        theta_n and theta_m are elements of the vector theta
load sstar0_vec 

q      =  abs(theta(1))./(1 + abs(theta(1)));
R_vec  = [abs(theta(2)) abs(theta(3)) abs(theta(6))  abs(theta(4)) ] ;  
lm     = abs(theta(5)); % lm in sector 1 in year 1984
ls     = abs(theta(7)); % ls 

quadlim =  fzero(@(z) 1-logncdf(z,lm,ls),exp(lm+ls^2/2));


for j = 1 :  length(lnX_normz) % loop across observations
    
    % load data
    ln_X    =  lnX_normz(j);
    phi     =  phi_data(j);
    std_eta =  std_eta_KT(j);   %   std  error of TFP regression (19)  (this is the new variable)

    
    % Find control  type (1 family, 2 holding, 3 foreing-institution, 4 public )
    R    =  R_vec(ctrl_vec(j))   ;  
     
    %              ****************
    sstar0  =  sstar0_vec(j);

    weight_X   = nobs_vec(j)/Sigma2(1);
    weight_phi = nobs_vec(j)/Sigma2(2);

    %  [X_hat,phi_hat, Xo, sstar] = fun_Solve_Xphi2(R,q,rho,lm,ls,bbeta,quadlim,sstar0);
   [X_hat,phi_hat,mu,Xo,std_Xo,std_X_unc,std_eps, sstar]=fun_Solve_Xphi_new(R,q,rho,lm,ls,bbeta,quadlim); 
    std_eta_hat =       std_eps/mu ;             
      
    % save errors of fit
    err(j,1)=( ln_X  - log(X_hat) );
    err(j,2)=(  phi  -  phi_hat    );
    err(j,3)=(  std_eta  -  std_eta_hat    );

    % benchmark values for derivatives: f(xo)
    f0(1) = log(X_hat)  ;
    f0(2) = phi_hat     ;
    f0(3) = std_eta_hat ;

    % loop for f(xo (1+eps))
    epsilon  = 0.1; % small step value to take numerical derivatives

    for k = 1: length(theta) % loop Delta for numerical derivatives

        theta(k) = (1+epsilon)*theta(k); % delta of argument

        q      = abs(theta(1))./(1 + abs(theta(1)));
        R_vec  = [abs(theta(2)) abs(theta(3)) abs(theta(6))  abs(theta(4)) ] ;
        R      =  R_vec(ctrl_vec(j))   ;
        lm     = abs(theta(5)); % lm in sector 1 in year 1984
        ls     = abs(theta(7)); % ls 
        
   %     [X_hat,phi_hat, Xo, sstar] = fun_Solve_Xphi2(R,q,rho,lm,ls,bbeta,quadlim,sstar0);
      [X_hat,phi_hat,mu,Xo,std_Xo,std_X_unc,std_eps, sstar]=fun_Solve_Xphi_new(R,q,rho,lm,ls,bbeta,quadlim);
      std_eta_hat = std_eps/mu ;
      f1(1) = log(X_hat);   f1(2) = phi_hat;  f1(3) = std_eta_hat ;
        
        % Build right-derivative at + epsilon
        dlnX_dt1(j,k) = (f1(1) - f0(1)) / ( epsilon*theta(k) );
        dphi_dt1(j,k) = (f1(2) - f0(2)) / ( epsilon*theta(k) );
        deta_dt1(j,k) = (f1(3) - f0(3)) / ( epsilon*theta(k) );

        % Compute   values at theta *(1- epsilon)
        theta(k) = theta(k)/(1+epsilon)*(1-epsilon);
        q      = abs(theta(1))./(1 + abs(theta(1)));
        R_vec  = [abs(theta(2)) abs(theta(3)) abs(theta(6))  abs(theta(4)) ] ;
        R      =  R_vec(ctrl_vec(j))   ;
        lm     = abs(theta(5)); % lm in sector 1 in year 1984
        ls     = abs(theta(7)); % ls 

     [X_hat,phi_hat,mu,Xo,std_Xo,std_X_unc,std_eps, sstar]=fun_Solve_Xphi_new(R,q,rho,lm,ls,bbeta,quadlim);
       std_eta_hat = std_eps/mu ;
 
        f2(1) = log(X_hat);   f2(2) = phi_hat ;   f2(3) = std_eta_hat ;
        
        % Build left derivative at - epsilon
        dlnX_dt2(j,k) = (f2(1) - f0(1)) / ( -epsilon*theta(k) );
        dphi_dt2(j,k) = (f2(2) - f0(2)) / ( -epsilon*theta(k) );
        deta_dt2(j,k) = (f2(3) - f0(3)) / ( -epsilon*theta(k) );

        % Build the derivative as an avg of the two
        dlnX_dt(j,k)  =  0.5* dlnX_dt1(j,k) + 0.5* dlnX_dt2(j,k); 
        dphi_dt(j,k)  =  0.5* dphi_dt1(j,k) + 0.5* dphi_dt2(j,k); 
        deta_dt(j,k)  =  0.5* deta_dt1(j,k) + 0.5* deta_dt2(j,k); 

        % Build the score Matrix
        GG_hat(j,k)  =  weight_X *err(j,1)*dlnX_dt(j,k) +  weight_phi *err(j,2)*dphi_dt(j,k) +   weight_phi *err(j,3)*deta_dt(j,k)   ;

        % Reset theta and implied values to baseline
        theta(k) = theta(k)/(1-epsilon);
        q      = abs(theta(1))./(1 + abs(theta(1)));
        R_vec  = [abs(theta(2)) abs(theta(3)) abs(theta(6))  abs(theta(4)) ] ;
        R      =  R_vec(ctrl_vec(j))   ;
        lm     = abs(theta(5)); % lm in sector 1 in year 1984
        ls     = abs(theta(7)); % ls 

    end% for k
    %j
end% for j

InfoM_BHHH = GG_hat'*GG_hat; 

% Compute asymptotic Info
InfoM = zeros(length(theta),length(theta));
for j = 1 :  length(lnX_normz) % loop for all observations
      weight_X   = nobs_vec(j)/Sigma2(1); 
      weight_phi = nobs_vec(j)/Sigma2(2); 
    for k = 1 : length(theta)
    for s = 1 : length(theta)
        Info_j(k,s) = weight_X*dlnX_dt(j,k)*dlnX_dt(j,s)+ weight_phi*dphi_dt(j,k)*dphi_dt(j,s)+ weight_phi*deta_dt(j,k)*deta_dt(j,s) ;
    end % s = 1 : length(theta)
    end % for k = 1 : length(theta)
    
    InfoM = InfoM + Info_j;
    
end % for j = 1 :  length(data(:,1)) % loop for all observations
