function [X,phi, mu , Xo,  std_Xo, std_X_unc, std_eps, sstar] = fun_Solve_Xphi_new(R,q,rho,lm,ls,bbeta,quadlim)
% Computes [X,phi, Xo, std_Xo, std_X_unc, std_eps, sstar] given the  fundamentals: q,R,G(ls,lm),bbeta

mu=exp(lm+ls^2/2);

% find sbar
options =optimset('TolX',1e-10,'TolFun',1e-10,'MaxFunEvals',100000,'MaxIter',100000,'Display','off');
[sstar,Hzero,flagH] = fzero( @(x) fun_H(x,q,R,rho,lm,ls,bbeta,quadlim),2*mu,options)     ;

% St st fraction of old -- phi in (0,1)
Fsbar = F_cdf(sstar,q,R,lm,ls);
phi  =  (1 + rho/(1 - Fsbar ) )^(-1);

%  mean productivity of senior executive
Xo     = (q * quad(@(s) fun_INT_G(s,lm,ls),max(1e-8,sstar-R),quadlim) + (1-q) * quad(@(s) fun_INT_G(s,lm,ls),sstar,quadlim) )...
                   / (1 -  Fsbar );   
               
% unconditional variance
std_X_unc = sqrt(exp(ls^2-1)*exp(2*lm+ls^2));

%  std dev of productivity of senior executive               
std_Xo = sqrt((q * quad(@(s) (s-Xo).^2 .* lognpdf(s,lm,ls),max(1e-8,sstar-R),quadlim) + (1-q) * quad(@(s) (s-Xo).^2 .* lognpdf(s,lm,ls),sstar,quadlim) )...
                   / (1 -  Fsbar ));               

%  std deviation of the innovation ``epsilon'' of  equation 18 proposition 3  appendix A3             
std_eps =  sqrt( phi^2*std_Xo^2 + (1-phi)^2*std_X_unc^2  ) ; 
           
%  Notice that the std deviation of the regression residual in equation 19
%  (IN LOGS )  are given by   eta_i = epsilon_i / mu   so that the variance
%  std_eta = std_eps / mu
 

% mean X               
X = phi * Xo + (1-phi)*mu; 